@@ -32,12 +32,17 @@
 #define MAX(a,b) ((a) > (b) ? (a) : (b))
 #define MIN(a,b) ((a) < (b) ? (a) : (b))
 
-#define PAD_TRUNCATE_SYMBOL    "\xe2\x80\xa6" /* symbol: "ellipsis" */
+#ifndef SFEED_DUMBTERM
 #define SCROLLBAR_SYMBOL_BAR   "\xe2\x94\x82" /* symbol: "light vertical" */
 #define SCROLLBAR_SYMBOL_TICK  " "
 #define LINEBAR_SYMBOL_BAR     "\xe2\x94\x80" /* symbol: "light horizontal" */
 #define LINEBAR_SYMBOL_RIGHT   "\xe2\x94\xa4" /* symbol: "light vertical and left" */
-#define UTF_INVALID_SYMBOL     "\xef\xbf\xbd" /* symbol: "replacement" */
+#else
+#define SCROLLBAR_SYMBOL_BAR   "|" /* symbol: "light vertical" */
+#define SCROLLBAR_SYMBOL_TICK  " "
+#define LINEBAR_SYMBOL_BAR     "-" /* symbol: "light horizontal" */
+#define LINEBAR_SYMBOL_RIGHT   "|" /* symbol: "light vertical and left" */
+#endif
 
 /* color-theme */
 #ifndef SFEED_THEME
@@ -79,7 +84,7 @@ struct pane {
 	int hidden; /* is visible or not */
 	int dirty; /* needs draw update */
 	/* (optional) callback functions */
-	struct row *(*row_get)(struct pane *, off_t pos);
+	struct row *(*row_get)(struct pane *, off_t);
 	char *(*row_format)(struct pane *, struct row *);
 	int (*row_match)(struct pane *, struct row *, const char *);
 };
@@ -203,12 +208,12 @@ ttywrite(const char *s)
 	return write(1, s, strlen(s));
 }
 
-/* hint for compilers and static analyzers that a function exits */
+/* Hint for compilers and static analyzers that a function exits. */
 #ifndef __dead
 #define __dead
 #endif
 
-/* print to stderr, call cleanup() and _exit(). */
+/* Print to stderr, call cleanup() and _exit(). */
 __dead void
 die(const char *fmt, ...)
 {
@@ -260,16 +265,17 @@ estrdup(const char *s)
 	return p;
 }
 
-/* wrapper for tparm which allows NULL parameter for str. */
+/* Wrapper for tparm which allows NULL parameter for str. */
 char *
-tparmnull(const char *str, long p1, long p2, long p3, long p4, long p5,
-          long p6, long p7, long p8, long p9)
+tparmnull(char *str, long p1, long p2, long p3, long p4, long p5, long p6,
+          long p7, long p8, long p9)
 {
 	if (!str)
 		return NULL;
 	return tparm(str, p1, p2, p3, p4, p5, p6, p7, p8, p9);
 }
 
+/* Counts column width of character string. */
 size_t
 colw(const char *s)
 {
@@ -300,7 +306,7 @@ colw(const char *s)
 	return col;
 }
 
-/* Format `len' columns of characters. If string is shorter pad the rest
+/* Format `len` columns of characters. If string is shorter pad the rest
    with characters `pad`. */
 int
 utf8pad(char *buf, size_t bufsiz, const char *s, size_t len, int pad)
@@ -1000,11 +1006,15 @@ lineeditor(void)
 			switch (sigstate) {
 			case 0:
 			case SIGWINCH:
-				continue; /* process signals later */
+				/* continue editing: process signal later */
+				continue;
 			case SIGINT:
-				sigstate = 0; /* exit prompt, do not quit */
-			case SIGTERM:
-				break; /* exit prompt and quit */
+				/* cancel prompt, but do not quit */
+				sigstate = 0; /* reset: do not handle it */
+				break;
+			default: /* other: SIGHUP, SIGTERM */
+				/* cancel prompt and handle signal after */
+				break;
 			}
 			free(input);
 			return NULL;
@@ -1196,7 +1206,7 @@ updatenewitems(struct feed *f)
 	p = &panes[PaneItems];
 	f->totalnew = 0;
 	for (i = 0; i < p->nrows; i++) {
-		row = &(p->rows[i]); /* do not use pane_row_get */
+		row = &(p->rows[i]); /* do not use pane_row_get() */
 		item = row->data;
 		if (urlfile)
 			item->isnew = urls_isnew(item->matchnew);
@@ -1224,7 +1234,7 @@ feed_load(struct feed *f, FILE *fp)
 	free(p->rows);
 	p->rows = ecalloc(sizeof(p->rows[0]), items.len + 1);
 	for (i = 0; i < items.len; i++)
-		p->rows[i].data = &(items.items[i]); /* do not use pane_row_get */
+		p->rows[i].data = &(items.items[i]); /* do not use pane_row_get() */
 
 	updatenewitems(f);
 
@@ -1409,12 +1419,14 @@ feed_plumb_selected_item(struct pane *p, int field)
 {
 	struct row *row;
 	struct item *item;
+	char *cmd[] = { plumbercmd, NULL, NULL };
 
 	if (!(row = pane_row_get(p, p->pos)))
 		return;
-	item = row->data;
 	markread(p, p->pos, p->pos, 1);
-	forkexec((char *[]) { plumbercmd, item->fields[field], NULL }, plumberia);
+	item = row->data;
+	cmd[1] = item->fields[field]; /* set first argument for plumber */
+	forkexec(cmd, plumberia);
 }
 
 void
@@ -1627,7 +1639,8 @@ mousereport(int button, int release, int keymask, int x, int y)
 {
 	struct pane *p;
 	size_t i;
-	int changedpane, dblclick, pos;
+	off_t pos;
+	int changedpane, dblclick;
 
 	if (!usemouse || release || button == -1)
 		return;
@@ -1664,7 +1677,7 @@ mousereport(int button, int release, int keymask, int x, int y)
 		selpane = i;
 		/* relative position on screen */
 		pos = y - p->y + p->pos - (p->pos % p->height);
-		dblclick = (pos == p->pos); /* clicking the same row twice */
+		dblclick = (pos == p->pos); /* clicking the already selected row */
 
 		switch (button) {
 		case 0: /* left-click */
@@ -1839,7 +1852,7 @@ markread(struct pane *p, off_t from, off_t to, int isread)
 			die("popen: %s", cmd);
 
 		for (i = from; i <= to && i < p->nrows; i++) {
-			/* do not use pane_row_get: no need for lazyload */
+			/* do not use pane_row_get(): no need for lazyload */
 			row = &(p->rows[i]);
 			item = row->data;
 			if (item->isnew != isnew) {
@@ -1886,7 +1899,8 @@ urls_cmp(const void *v1, const void *v2)
 int
 urls_isnew(const char *url)
 {
-	return bsearch(&url, urls, nurls, sizeof(char *), urls_cmp) == NULL;
+	return (!nurls ||
+	       bsearch(&url, urls, nurls, sizeof(char *), urls_cmp) == NULL);
 }
 
 void
@@ -1928,7 +1942,8 @@ urls_read(void)
 	fclose(fp);
 	free(line);
 
-	qsort(urls, nurls, sizeof(char *), urls_cmp);
+	if (nurls > 0)
+		qsort(urls, nurls, sizeof(char *), urls_cmp);
 }
 
 int
@@ -1937,10 +1952,9 @@ main(int argc, char *argv[])
 	struct pane *p;
 	struct feed *f;
 	struct row *row;
-	size_t i;
 	char *name, *tmp;
 	char *search = NULL; /* search text */
-	int button, ch, fd, keymask, release, x, y;
+	int button, ch, fd, i, keymask, release, x, y;
 	off_t pos;
 
 #ifdef __OpenBSD__
@@ -2119,17 +2133,11 @@ main(int argc, char *argv[])
 			break;
 keyup:
 		case 'k':
-		case 'K':
 			pane_scrolln(&panes[selpane], -1);
-			if (ch == 'K')
-				goto openitem;
 			break;
 keydown:
 		case 'j':
-		case 'J':
 			pane_scrolln(&panes[selpane], +1);
-			if (ch == 'J')
-				goto openitem;
 			break;
 keyleft:
 		case 'h':
@@ -2147,6 +2155,28 @@ keyright:
 			if (layout == LayoutMonocle)
 				updategeom();
 			break;
+		case 'K':
+			p = &panes[selpane];
+			if (!p->nrows)
+				break;
+			for (pos = p->pos - 1; pos >= 0; pos--) {
+				if ((row = pane_row_get(p, pos)) && row->bold) {
+					pane_setpos(p, pos);
+					break;
+				}
+			}
+			break;
+		case 'J':
+			p = &panes[selpane];
+			if (!p->nrows)
+				break;
+			for (pos = p->pos + 1; pos < p->nrows; pos++) {
+				if ((row = pane_row_get(p, pos)) && row->bold) {
+					pane_setpos(p, pos);
+					break;
+				}
+			}
+			break;
 		case '\t':
 			selpane = selpane == PaneFeeds ? PaneItems : PaneFeeds;
 			if (layout == LayoutMonocle)
@@ -2254,7 +2284,6 @@ nextpage:
 			break;
 		case 'o': /* feeds: load, items: plumb URL */
 		case '\n':
-openitem:
 			if (selpane == PaneFeeds && panes[selpane].nrows)
 				feed_open_selected(&panes[selpane]);
 			else if (selpane == PaneItems && panes[selpane].nrows)
