@@ -11,8 +11,8 @@ static size_t linesize;
 static char host[256], *user, dtimebuf[32], mtimebuf[32];
 static int usecontent = 0; /* env variable: $SFEED_MBOX_CONTENT */
 
-static unsigned long
-djb2(unsigned char *s, unsigned long hash)
+static unsigned long long
+djb2(unsigned char *s, unsigned long long hash)
 {
 	int c;
 
@@ -59,14 +59,14 @@ printfeed(FILE *fp, const char *feedname)
 	char *fields[FieldLast], timebuf[32];
 	struct tm parsedtm, *tm;
 	time_t parsedtime;
-	unsigned long hash;
+	unsigned long long hash;
 	ssize_t linelen;
 	int ishtml;
 
 	while ((linelen = getline(&line, &linesize, fp)) > 0) {
 		if (line[linelen - 1] == '\n')
 			line[--linelen] = '\0';
-		hash = djb2((unsigned char *)line, 5381UL);
+		hash = djb2((unsigned char *)line, 5381ULL);
 		parseline(line, fields);
 
 		/* mbox + mail header */
@@ -84,7 +84,7 @@ printfeed(FILE *fp, const char *feedname)
 		printf("From: %s <sfeed@>\n", fields[FieldAuthor][0] ? fields[FieldAuthor] : feedname);
 		printf("To: %s <%s@%s>\n", user, user, host);
 		printf("Subject: %s\n", fields[FieldTitle]);
-		printf("Message-ID: <%s%s%lu@%s>\n",
+		printf("Message-ID: <%s%s%llu@%s>\n",
 		       fields[FieldUnixTimestamp],
 		       fields[FieldUnixTimestamp][0] ? "." : "",
 		       hash, feedname);
@@ -104,14 +104,14 @@ printfeed(FILE *fp, const char *feedname)
 				fputs("Link: <a href=\"", stdout);
 				xmlencode(fields[FieldLink], stdout);
 				fputs("\">", stdout);
-				fputs(fields[FieldLink], stdout);
+				xmlencode(fields[FieldLink], stdout);
 				fputs("</a><br/>\n", stdout);
 			}
 			if (fields[FieldEnclosure][0]) {
 				fputs("Enclosure: <a href=\"", stdout);
 				xmlencode(fields[FieldEnclosure], stdout);
 				fputs("\">", stdout);
-				fputs(fields[FieldEnclosure], stdout);
+				xmlencode(fields[FieldEnclosure], stdout);
 				fputs("</a><br/>\n", stdout);
 			}
 			fputs("</p>\n", stdout);
@@ -123,6 +123,11 @@ printfeed(FILE *fp, const char *feedname)
 		}
 		if (usecontent) {
 			fputs("\n", stdout);
+			if (ishtml && fields[FieldLink][0]) {
+				fputs("<base href=\"", stdout);
+				xmlencode(fields[FieldLink], stdout);
+				fputs("\"/>\n", stdout);
+			}
 			printcontent(fields[FieldContent], stdout);
 		}
 		fputs("\n\n", stdout);
