@@ -66,7 +66,7 @@ strcasestr(const char *h, const char *n)
 	return NULL;
 }
 
-/* check if string has a non-empty scheme / protocol part */
+/* Check if string has a non-empty scheme / protocol part. */
 int
 uri_hasscheme(const char *s)
 {
@@ -79,6 +79,8 @@ uri_hasscheme(const char *s)
 	return (*p == ':' && p != s);
 }
 
+/* Parse URI string `s` into an uri structure `u`.
+   Returns 0 on success or -1 on failure */
 int
 uri_parse(const char *s, struct uri *u)
 {
@@ -298,8 +300,9 @@ strtotime(const char *s, time_t *t)
 	l = strtoll(s, &e, 10);
 	if (errno || *s == '\0' || *e)
 		return -1;
-	/* NOTE: assumes time_t is 64-bit on 64-bit platforms:
-	         long long (at least 32-bit) to time_t. */
+
+	/* NOTE: the type long long supports the 64-bit range. If time_t is
+	   64-bit it is "2038-ready", otherwise it is truncated/wrapped. */
 	if (t)
 		*t = (time_t)l;
 
@@ -322,7 +325,7 @@ xmlencode(const char *s, FILE *fp)
 	}
 }
 
-/* print `len' columns of characters. If string is shorter pad the rest with
+/* print `len` columns of characters. If string is shorter pad the rest with
  * characters `pad`. */
 void
 printutf8pad(FILE *fp, const char *s, size_t len, int pad)
@@ -351,11 +354,11 @@ printutf8pad(FILE *fp, const char *s, size_t len, int pad)
 			}
 
 			if (col + w > len || (col + w == len && s[i + inc])) {
-				fputs("\xe2\x80\xa6", fp); /* ellipsis */
+				fputs(PAD_TRUNCATE_SYMBOL, fp); /* ellipsis */
 				col++;
 				break;
 			} else if (rl < 0) {
-				fputs("\xef\xbf\xbd", fp); /* replacement */
+				fputs(UTF_INVALID_SYMBOL, fp); /* replacement */
 				col++;
 				continue;
 			}
@@ -364,7 +367,7 @@ printutf8pad(FILE *fp, const char *s, size_t len, int pad)
 		} else {
 			/* optimization: simple ASCII character */
 			if (col + 1 > len || (col + 1 == len && s[i + 1])) {
-				fputs("\xe2\x80\xa6", fp); /* ellipsis */
+				fputs(PAD_TRUNCATE_SYMBOL, fp); /* ellipsis */
 				col++;
 				break;
 			}
